from datetime import datetime

from django.contrib.auth import login
from django.core.exceptions import ObjectDoesNotExist
from django.template import loader, RequestContext
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _


class BearerAuthHandler():
    """Handles Bearer token authentication calls.

    SPEC: http://tools.ietf.org/html/rfc6750

    """

    _token = None
    _request = None
    _response = None
    _error = None
    _scope = None

    def __init__(self, request, scope):
        self._request = request
        self._scope = scope

        self.fetch_token()
        self.validate_token()
        self.prepare_response()

    def fetch_token(self):
        token = None

        # Authorization Request Header Field
        authorization_method = self._request.META.get('HTTP_AUTHORIZATION')
        if authorization_method is not None:
            auth_method_type, auth_method_value = authorization_method.split(' ', 1)
            if auth_method_type == 'Bearer':
                token = auth_method_value
        else:
            # Form-Encoded Body Parameter or URI Query Parameter
            token = self._request.REQUEST.get('access_token')

        if token is None:
            self._error = 'invalid_request'
        else:
            self._token = token

    def validate_token(self):

        if self._token is None:
            return False

        from oauthost.models import Token
        try:
            token = Token.objects.get(access_token=self._token)
        except ObjectDoesNotExist:
            self._error = 'invalid_token'
            return False

        # If token found is granted to all the different token type.
        if token.access_token_type != 'bearer':
            self._error = 'invalid_token'
            return False

        # Token has expired.
        if token.expires_at is not None and token.expires_at <= datetime.now():
            self._error = 'invalid_token'
            return False

        # If target scope is defined, let's verify that the token has access to it.
        if self._scope is not None:
            if not token.scopes.filter(identifier=self._scope).count():
                self._error = 'insufficient_scope'
                return False

        # Token is valid and now we'll log it's owner in.

        # Manual .backend attribute is set as an alternative to `authenticate()`.
        # For now this hardcoded backend will do.
        token.user.backend = 'django.contrib.auth.backends.ModelBackend'
        login(self._request, token.user)

        return True

    def prepare_response(self):

        if self._error is not None:
            from .settings import TEMPLATE_RESTRICTED

            errors = {
                'invalid_request': (400, 'Request is malformed. Check request parameters validity.'),
                'invalid_token': (401, 'Given access token is invalid'),
                'insufficient_scope': (403, 'Access token grants no access to required scope.')
            }

            current_error = errors[self._error]
            additional_params = {
                'error': self._error, 'error_description': current_error[1]
            }
            additional_params = ',' . join( [ '%s="%s"' % (i[0], i[1]) for i in additional_params.items() ]  )
            context = RequestContext(self._request)
            self._response = HttpResponse(content=loader.render_to_string(TEMPLATE_RESTRICTED,
                    {'oauthost_title': _('Access Restricted')}, context), status=current_error[0])
            self._response['WWW-Authenticate'] = 'Bearer %s' % additional_params

    def response(self):
        return self._response


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Scope'
        db.create_table('oauthost_scope', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('identifier', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=250)),
        ))
        db.send_create_signal('oauthost', ['Scope'])

        # Adding model 'Client'
        db.create_table('oauthost_client', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date_registered', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(unique=True, max_length=100)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('description', self.gf('django.db.models.fields.TextField')(max_length=100)),
            ('link', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('identifier', self.gf('django.db.models.fields.CharField')(unique=True, max_length=250, blank=True)),
            ('token_lifetime', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('password', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
            ('type', self.gf('django.db.models.fields.IntegerField')(default=1)),
            ('hash_sign_supported', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal('oauthost', ['Client'])

        # Adding M2M table for field scopes on 'Client'
        db.create_table('oauthost_client_scopes', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('client', models.ForeignKey(orm['oauthost.client'], null=False)),
            ('scope', models.ForeignKey(orm['oauthost.scope'], null=False))
        ))
        db.create_unique('oauthost_client_scopes', ['client_id', 'scope_id'])

        # Adding model 'RedirectionEndpoint'
        db.create_table('oauthost_redirectionendpoint', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauthost.Client'])),
            ('uri', self.gf('django.db.models.fields.URLField')(max_length=200)),
        ))
        db.send_create_signal('oauthost', ['RedirectionEndpoint'])

        # Adding model 'AuthorizationCode'
        db.create_table('oauthost_authorizationcode', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date_issued', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=7)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauthost.Client'])),
            ('uri', self.gf('django.db.models.fields.URLField')(max_length=200)),
        ))
        db.send_create_signal('oauthost', ['AuthorizationCode'])

        # Adding M2M table for field scopes on 'AuthorizationCode'
        db.create_table('oauthost_authorizationcode_scopes', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('authorizationcode', models.ForeignKey(orm['oauthost.authorizationcode'], null=False)),
            ('scope', models.ForeignKey(orm['oauthost.scope'], null=False))
        ))
        db.create_unique('oauthost_authorizationcode_scopes', ['authorizationcode_id', 'scope_id'])

        # Adding model 'Token'
        db.create_table('oauthost_token', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('date_issued', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('expires_at', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('access_token', self.gf('django.db.models.fields.CharField')(unique=True, max_length=32)),
            ('refresh_token', self.gf('django.db.models.fields.CharField')(max_length=32, unique=True, null=True, blank=True)),
            ('access_token_type', self.gf('django.db.models.fields.CharField')(default='bearer', max_length=100)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('client', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauthost.Client'])),
            ('code', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['oauthost.AuthorizationCode'], null=True, blank=True)),
        ))
        db.send_create_signal('oauthost', ['Token'])

        # Adding M2M table for field scopes on 'Token'
        db.create_table('oauthost_token_scopes', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('token', models.ForeignKey(orm['oauthost.token'], null=False)),
            ('scope', models.ForeignKey(orm['oauthost.scope'], null=False))
        ))
        db.create_unique('oauthost_token_scopes', ['token_id', 'scope_id'])


    def backwards(self, orm):
        
        # Deleting model 'Scope'
        db.delete_table('oauthost_scope')

        # Deleting model 'Client'
        db.delete_table('oauthost_client')

        # Removing M2M table for field scopes on 'Client'
        db.delete_table('oauthost_client_scopes')

        # Deleting model 'RedirectionEndpoint'
        db.delete_table('oauthost_redirectionendpoint')

        # Deleting model 'AuthorizationCode'
        db.delete_table('oauthost_authorizationcode')

        # Removing M2M table for field scopes on 'AuthorizationCode'
        db.delete_table('oauthost_authorizationcode_scopes')

        # Deleting model 'Token'
        db.delete_table('oauthost_token')

        # Removing M2M table for field scopes on 'Token'
        db.delete_table('oauthost_token_scopes')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'oauthost.authorizationcode': {
            'Meta': {'object_name': 'AuthorizationCode'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauthost.Client']"}),
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '7'}),
            'date_issued': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'uri': ('django.db.models.fields.URLField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauthost.client': {
            'Meta': {'object_name': 'Client'},
            'date_registered': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '100'}),
            'hash_sign_supported': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '250', 'blank': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'token_lifetime': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'oauthost.redirectionendpoint': {
            'Meta': {'object_name': 'RedirectionEndpoint'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauthost.Client']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uri': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        'oauthost.scope': {
            'Meta': {'ordering': "['title']", 'object_name': 'Scope'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        'oauthost.token': {
            'Meta': {'object_name': 'Token'},
            'access_token': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'access_token_type': ('django.db.models.fields.CharField', [], {'default': "'bearer'", 'max_length': '100'}),
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauthost.Client']"}),
            'code': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['oauthost.AuthorizationCode']", 'null': 'True', 'blank': 'True'}),
            'date_issued': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'expires_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refresh_token': ('django.db.models.fields.CharField', [], {'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['oauthost']



class OauthostException(Exception):
    """Base oauthost exceptions. All the others should inherit from it."""


class EndpointError(OauthostException):
    """Base exception for conflicts occured at endpoints: TokenEndpoint, AuthorizeEndpoint."""

    def as_response(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'AuthorizationCode.uri'
        db.alter_column(u'oauthost_authorizationcode', 'uri', self.gf('oauthost.fields.URLSchemeField')(max_length=200))

        # Changing field 'RedirectionEndpoint.uri'
        db.alter_column(u'oauthost_redirectionendpoint', 'uri', self.gf('oauthost.fields.URLSchemeField')(max_length=200))
        # Adding field 'Scope.status'
        db.add_column(u'oauthost_scope', 'status',
                      self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True),
                      keep_default=False)


    def backwards(self, orm):

        # Changing field 'AuthorizationCode.uri'
        db.alter_column(u'oauthost_authorizationcode', 'uri', self.gf('django.db.models.fields.URLField')(max_length=200))

        # Changing field 'RedirectionEndpoint.uri'
        db.alter_column(u'oauthost_redirectionendpoint', 'uri', self.gf('django.db.models.fields.URLField')(max_length=200))
        # Deleting field 'Scope.status'
        db.delete_column(u'oauthost_scope', 'status')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'oauthost.authorizationcode': {
            'Meta': {'object_name': 'AuthorizationCode'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['oauthost.Client']"}),
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '7'}),
            'date_issued': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'uri': ('oauthost.fields.URLSchemeField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'oauthost.client': {
            'Meta': {'object_name': 'Client'},
            'date_registered': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '100'}),
            'hash_sign_supported': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '250', 'blank': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'token_lifetime': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'oauthost.redirectionendpoint': {
            'Meta': {'object_name': 'RedirectionEndpoint'},
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['oauthost.Client']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'uri': ('oauthost.fields.URLSchemeField', [], {'max_length': '200'})
        },
        u'oauthost.scope': {
            'Meta': {'ordering': "['title']", 'object_name': 'Scope'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'status': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        u'oauthost.token': {
            'Meta': {'object_name': 'Token'},
            'access_token': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'access_token_type': ('django.db.models.fields.CharField', [], {'default': "'bearer'", 'max_length': '100'}),
            'client': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['oauthost.Client']"}),
            'code': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['oauthost.AuthorizationCode']", 'null': 'True', 'blank': 'True'}),
            'date_issued': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'expires_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refresh_token': ('django.db.models.fields.CharField', [], {'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'scopes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['oauthost.Scope']", 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['oauthost']


from functools import wraps

from django.utils.decorators import available_attrs

from .utils import auth_handler_response


def oauth_required(scope=None, scope_auto=False):
    """Views decorator checking user oauth token.

    ``scope`` - scope identifier string to check token has access to the scope.
    ``scope_auto`` - if *True* scope identifier will be built automatically
        using the following format: '<application_name>:<decorated_view_name>'.
        E.g.: for application named *polls* and having *detail* view scope
        would be *polls:detail*.

    """
    def decorated_view(view_function):
        @wraps(view_function, assigned=available_attrs(view_function))
        def wrapper(request, *args, **kwargs):
            target_scope = scope

            if scope_auto:
                target_scope = '%(app_name)s:%(view_name)s' % {
                    'app_name': view_function.__module__.split('.')[0],
                    'view_name': view_function.__name__}

            return auth_handler_response(request, scope=target_scope) or view_function(request, *args, **kwargs)
        return wrapper

    return decorated_view



from os import path
from inspect import getfile

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.db import IntegrityError

from oauthost.models import Scope


class Command(BaseCommand):

    args = '<app_name app_name ...>'
    help = '''Registers OAuth2 scopes from application views in form of `<application_name>:<decorated_view_name>`.
It requires views.py to reside in application directory. Use @oauth_required to decorate view which requires scope syncing.'''

    def handle(self, *args, **options):

        if not len(args):
            raise CommandError('This command accepts space delimited list of application names.')

        if not set(args).issubset(settings.INSTALLED_APPS):
            raise CommandError('One or more application names issued to the command are not in INSTALLED_APPS.')

        for app_name in args:

            decorated_views_count = 0

            self.stdout.write('Working on "%s" application ...\n' % app_name)
            try:
                app_views = __import__('%s.views' % app_name)
            except ImportError:
                raise CommandError('No views.py found in the application.')

            app_views_substr = path.join('oauthost', 'decorators.py')

            for func_name in dir(app_views.views):
                if '__' not in func_name:
                    func = getattr(app_views.views, func_name)
                    # That's how we find decorated views.
                    if func_name != 'oauth_required' and app_views_substr in getfile(func):
                        decorated_views_count += 1
                        # TODO That would be nice to have here a value of `scope` parameter of @oauth_required if it set.
                        # That is, of course, if only we can trace it up at a low cost.
                        scope_name = '%(app_name)s:%(view_name)s' % {'app_name': app_name, 'view_name': func_name}
                        self.stdout.write('    Found "%s" view. Syncing "%s" scope ... ' % (func_name, scope_name))
                        # A try to give our scope a pretty name.
                        scope_title = '%s %s' % (app_name.capitalize(), ' '.join([word.capitalize() for word in func_name.split('_')]))
                        scope = Scope(identifier=scope_name, title=scope_title)
                        try:
                            scope.save()
                        except IntegrityError:
                            self.stdout.write('WARNING: Scope skipped as already exists\n')
                        else:
                            self.stdout.write('Done\n')

            if not decorated_views_count:
                self.stdout.write('NOTE: No views decorated with "@oauth_required" are found in the application.\n')

            self.stdout.write('\n')



import json

from django.test import TestCase
from django.test.client import Client as TestClient
from django.contrib.auth.models import User
from django.conf import settings

from .models import AuthorizationCode, Token, Client, RedirectionEndpoint, Scope
from .toolbox import register_client
from .exceptions import OauthostException


URL_TOKEN = '/token/'
URL_AUTHORIZE = '/auth/'


class OAuthostClient(TestClient):

    def post(self, path, data=None, **extra):
        response = super(OAuthostClient, self).post(path, data=data, **extra)
        if path == URL_TOKEN:
            response.content_json = json.loads(response.content.decode('utf-8'))
        return response


def parse_location_header(response, use_uri_fragment=False):
    delimiter = '?'
    if use_uri_fragment:
        delimiter = '#'
    query = response['Location'].split(delimiter)[1]
    query = query.split('&')
    parsed = {}
    for part in query:
        key, value = part.split('=')
        parsed[key] = value
    return parsed


class ToolboxCheck(TestCase):

    def test_register_client(self):
        user = User(username='Fred')
        user.set_password('12345')
        user.save()

        scope1 = Scope(identifier='scope1')
        scope1.save()

        cl = register_client('client1_title', 'client1', 'http://client1url.com/client1/', user)
        self.assertEqual(cl.identifier, 'client1')
        self.assertEqual(cl.title, 'client1_title')
        self.assertEqual(cl.user, user)
        uris = cl.redirection_uris.all()
        self.assertEqual(len(uris), 1)
        self.assertEqual(uris[0].uri, 'http://client1url.com/client1/')

        self.assertRaises(OauthostException, register_client, 'client2_title', 'client2', 'http://client2url.com/client2/', user, scopes_list=[scope1, 'scope2'], register_unknown_scopes=False)

        cl = register_client('client2_title', 'client2', 'http://client2url.com/client2/', user, scopes_list=[scope1, 'scope2'], token_lifetime=300, public=False, client_params={'description': 'client2_decr'})
        self.assertEqual(cl.identifier, 'client2')
        self.assertEqual(cl.title, 'client2_title')
        self.assertEqual(cl.token_lifetime, 300)
        self.assertEqual(cl.user, user)
        self.assertEqual(cl.description, 'client2_decr')
        self.assertNotEqual(cl.type, Client.TYPE_PUBLIC)
        self.assertEqual(len(cl.scopes.all()), 2)
        uris = cl.redirection_uris.all()
        self.assertEqual(len(uris), 1)
        self.assertEqual(uris[0].uri, 'http://client2url.com/client2/')


class EndpointTokenCheck(TestCase):

    client_class = OAuthostClient

    def test_grant_authorization_code(self):

        # Secure connection check
        settings.DEBUG = False
        resp = self.client.get(URL_TOKEN, {})
        self.assertEqual(resp.status_code, 403)
        settings.DEBUG = True

        resp = self.client.post(URL_TOKEN, {'grant_type': 'a'})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'unsupported_grant_type')

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient')
        client_1.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        code_1 = AuthorizationCode(user=user_1, client=client_1, uri=redirect_1.uri)
        code_1.save()

        Scope(identifier='scope1').save()

        # Missing client authentication data.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1'})
        self.assertEqual(resp.status_code, 401)
        self.assertEqual(resp.content_json['error'], 'invalid_client')

        # Missing all required params.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'client_id': client_1.identifier,
                                             'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_request')

        # Missing redirect URI.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': 'wrong_code',
                                             'client_id': client_1.identifier, 'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_request')

        # Missing code.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'redirect_uri': 'http://wrong-url.com',
                                             'client_id': client_1.identifier, 'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_request')

        # Wrong code.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': 'invalid',
                                             'redirect_uri': 'http://localhost:8000/abc/',
                                             'client_id': client_1.identifier, 'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')

        # Wrong URI.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': code_1.code,
                                             'redirect_uri': 'http://wrong-url.com/', 'client_id': client_1.identifier,
                                             'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')

        # Valid call for a token.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': code_1.code,
                                             'redirect_uri': redirect_1.uri, 'client_id': client_1.identifier,
                                             'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)

        # An additional call for code issues token and code invalidation.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': '1234567',
                                             'redirect_uri': 'http://localhost:8000/abc/',
                                             'client_id': client_1.identifier, 'client_secret': client_1.password})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')


class EndpointAuthorizeCheck(TestCase):

    client_class = OAuthostClient

    def test_auth(self):

        # User is not logged in.
        resp = self.client.get(URL_AUTHORIZE, {'client_id': '100'})
        self.assertEqual(resp.status_code, 302)

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        # Logging the user in.
        self.client.login(username='Fred', password='12345')

        # Secure connection check
        settings.DEBUG = False
        resp = self.client.get(URL_AUTHORIZE, {})
        self.assertEqual(resp.status_code, 403)
        settings.DEBUG = True

        client_1 = Client(user=user_1, title='OClient', identifier='cl012345')
        client_1.save()

        client_2 = Client(user=user_1, title='OGOClient')
        client_2.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        redirect_2 = RedirectionEndpoint(client=client_2, uri='http://redirect-test1.com')
        redirect_2.save()

        redirect_3 = RedirectionEndpoint(client=client_2, uri='http://redirect-test2.com')
        redirect_3.save()

        Scope(identifier='scope1').save()

        # Missing client id.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1'})
        self.assertEqual(resp.status_code, 400)

        # Invalid client id.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'client_id': 'invalid'})
        self.assertEqual(resp.status_code, 400)

        # Client 2 - No redirect URI in request.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'client_id': client_2.identifier})
        self.assertEqual(resp.status_code, 400)

        # Client 2 - Unknown URI in request.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'redirect_uri': 'http://noitisnot.com', 'client_id': client_2.identifier})
        self.assertEqual(resp.status_code, 400)

        # Missing response type.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'client_id': client_1.identifier, 'state': 'abc', 'scope': 'scope1'})
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(parse_location_header(resp)['error'], 'unsupported_response_type')
        self.assertEqual(parse_location_header(resp)['state'], 'abc')

        # Wrong response type
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'client_id': client_1.identifier, 'response_type': 'habrahabr', 'state': 'abc', 'scope': 'scope1'})
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(parse_location_header(resp)['error'], 'unsupported_response_type')
        self.assertEqual(parse_location_header(resp)['state'], 'abc')

        # Valid code request.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'state': 'somestate', 'client_id': client_1.identifier})
        self.assertEqual(resp.status_code, 200)

        # User declines auth.
        resp = self.client.post(URL_AUTHORIZE, {'auth_decision': 'is_made'})
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(parse_location_header(resp)['error'], 'access_denied')

        # Again Valid code request.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'state': 'somestatetwo', 'client_id': client_1.identifier})
        self.assertEqual(resp.status_code, 200)

        # User confirms auth.
        resp = self.client.post(URL_AUTHORIZE, {'auth_decision': 'is_made', 'confirmed': 'yes'})
        self.assertEqual(resp.status_code, 302)
        self.assertIn('code', parse_location_header(resp))
        self.assertEqual(parse_location_header(resp)['state'], 'somestatetwo')

        # ============= Implicit grant tests.

        # Valid token request.
        self.client.login(username='Fred', password='12345')
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'token', 'scope': 'scope1', 'state': 'some_state_three', 'client_id': client_1.identifier})
        self.assertEqual(resp.status_code, 200)

        # User confirms token grant.
        resp = self.client.post(URL_AUTHORIZE, {'auth_decision': 'is_made', 'confirmed': 'yes'})
        self.assertEqual(resp.status_code, 302)
        params = parse_location_header(resp, True)
        self.assertIn('access_token', params)
        self.assertIn('token_type', params)
        self.assertEqual(params['state'], 'some_state_three')


class GrantsCheck(TestCase):

    client_class = OAuthostClient

    def test_authorization_code_unsafe(self):

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient')
        client_1.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        Scope(identifier='scope1').save()

        # Logging the user in.
        self.client.login(username='Fred', password='12345')

        # Valid code request.
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'client_id': client_1.identifier})
        self.assertEqual(resp.status_code, 200)

        # User confirms auth.
        resp = self.client.post(URL_AUTHORIZE, {'auth_decision': 'is_made', 'confirmed': 'yes'})
        self.assertEqual(resp.status_code, 302)
        params = parse_location_header(resp)
        self.assertIn('code', params)

        # Auth code given.
        code = params['code']

        # Valid token by code request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': code,
                                             'redirect_uri': redirect_1.uri,
                                             'client_id': client_1.identifier,
                                             'client_secret': client_1.password})

        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)

    def test_authorization_code_http_basic(self):

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient', identifier='OClient', password='cl012345')
        client_1.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        settings.DEBUG = True

        # Logging the user in.
        self.client.login(username='Fred', password='12345')

        Scope(identifier='scope1').save()

        # Valid code request.
        resp = self.client.get(URL_AUTHORIZE, {'response_type': 'code', 'scope': 'scope1', 'client_id': client_1.identifier})
        self.assertEqual(resp.status_code, 200)

        # User confirms auth.
        resp = self.client.post(URL_AUTHORIZE, {'auth_decision': 'is_made', 'confirmed': 'yes'})
        self.assertEqual(resp.status_code, 302)
        params = parse_location_header(resp)
        self.assertIn('code', params)

        # Auth code given.
        code = params['code']

        # Invalid token by code request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'code': code,
                                             'redirect_uri': redirect_1.uri},
                                Authorization='Basic Tqrqwer==')
        self.assertEqual(resp.status_code, 401)
        self.assertIn('www-authenticate', resp._headers)
        self.assertEqual(resp._headers['www-authenticate'][1], 'Basic')

        # Valid token by code request.
        # HTTP Basic data - OClient:cl012345 --> T0NsaWVudDpjbDAxMjM0NQ==
        resp = self.client.post(URL_TOKEN, {'grant_type': 'authorization_code', 'scope': 'scope1', 'code': code,
                                             'redirect_uri': redirect_1.uri},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)

    def test_scope(self):

        settings.DEBUG = True

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient1', identifier='OClient', password='cl012345')
        client_1.save()

        # Scope is missing.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'username': 'Fred', 'password': '12345'}, Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_scope')

        # No scope supported by server.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'username': 'Fred', 'password': '12345', 'scope': 'my scope'}, Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_scope')

        scope1 = Scope(identifier='scope1')
        scope1.save()
        scope2 = Scope(identifier='scope2')
        scope2.save()
        scope3 = Scope(identifier='scope3', status=Scope.STATUS_DISABLED)
        scope3.save()

        client_2 = Client(user=user_1, title='OClien2', identifier='OClient2', password='cl012345')
        client_2.save()
        client_2.scopes.add(scope2)

        # Unsupported (or disabled) client scope request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'username': 'Fred', 'password': '12345', 'scope': 'scope1 scope2'}, Authorization='Basic T0NsaWVudDI6Y2wwMTIzNDU=')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_scope')

        # Unsupported (or disabled) server scope request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'username': 'Fred', 'password': '12345', 'scope': 'scope1 scope3'}, Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_scope')

        # Unsupported scope request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'username': 'Fred', 'password': '12345', 'scope': 'scope1'}, Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        # print('****' * 20)
        # print(resp.content_json['error_description'])
        # print('****' * 20)
        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)
        self.assertEqual(resp.content_json['scope'], 'scope1')

    def test_token_by_user_credentials(self):

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient', identifier='OClient', password='cl012345')
        client_1.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        Scope(identifier='scope1').save()

        # Missing params.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'scope': 'scope1'}, Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_request')

        # Invalid params.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'scope': 'scope1', 'username': 'FalseUser', 'password': 'FalsePassword'},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')

        # Valid token by password request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'password', 'scope': 'scope1', 'username': 'Fred',
                                             'password': '12345'},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')

        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)
        self.assertFalse('expires_in' in resp.content_json)

    def test_token_by_client_credentials(self):

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient', identifier='OClient', password='cl012345')
        client_1.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        Scope(identifier='scope1').save()

        # Valid token by client credentials request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'client_credentials', 'scope': 'scope1'},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')

        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' not in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)

        access_token = resp.content_json['access_token']
        token = Token.objects.get(access_token=access_token)
        self.assertEqual(user_1, token.user)

    def test_refresh_token_http_basic(self):

        user_1 = User(username='Fred')
        user_1.set_password('12345')
        user_1.save()

        client_1 = Client(user=user_1, title='OClient', identifier='OClient', password='cl012345')
        client_1.save()

        client_2 = Client(user=user_1, title='OGOClient', identifier='OGOClient', password='cl543210')
        client_2.save()

        redirect_1 = RedirectionEndpoint(client=client_1, uri='http://redirect-test.com')
        redirect_1.save()

        token_1 = Token(client=client_1, user=user_1)
        token_1.save()

        token_2 = Token(client=client_2, user=user_1)
        token_2.save()

        date_issued = token_1.date_issued
        access_token = token_1.access_token
        refresh_token = token_1.refresh_token

        refresh_token_wrong_client = token_2.refresh_token

        # Missing required params.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'refresh_token'},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_request')

        # Invalid refresh token supplied.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'refresh_token', 'refresh_token': 'invalid'},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')

        # Refresh token from another client is supplied.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'refresh_token', 'refresh_token': refresh_token_wrong_client},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content_json['error'], 'invalid_grant')

        # Valid request.
        resp = self.client.post(URL_TOKEN, {'grant_type': 'refresh_token', 'refresh_token': refresh_token},
                                Authorization='Basic T0NsaWVudDpjbDAxMjM0NQ==')

        self.assertEqual(resp.status_code, 200)
        self.assertTrue('access_token' in resp.content_json)
        self.assertTrue('refresh_token' in resp.content_json)
        self.assertTrue('token_type' in resp.content_json)
        self.assertTrue('expires_in' not in resp.content_json)

        self.assertNotEqual(access_token, resp.content_json['access_token'])
        self.assertNotEqual(refresh_token, resp.content_json['refresh_token'])

        token_updated = Token.objects.get(access_token=resp.content_json['access_token'])
        self.assertNotEqual(date_issued, token_updated.date_issued)


# TODO Add tests for Bearer auth.



#! /usr/bin/env python
import sys
import os

from django.conf import settings


APP_NAME = 'oauthost'


def main():
    sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

    if not settings.configured:
        settings.configure(
            INSTALLED_APPS=('django.contrib.auth', 'django.contrib.contenttypes', 'django.contrib.sessions', APP_NAME),
            DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
            ROOT_URLCONF='oauthost.urls'
        )

    from django.test.utils import get_runner
    runner = get_runner(settings)()
    failures = runner.run_tests((APP_NAME,))

    sys.exit(failures)


if __name__ == '__main__':
    main()



from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required

from .endpoints import AuthorizeEndpoint, TokenEndpoint


@login_required
def endpoint_authorize(request):
    return AuthorizeEndpoint(request).get_response()


@csrf_exempt
def endpoint_token(request):
    return TokenEndpoint(request).get_response()



from uuid import uuid4
from random import randrange

from django import VERSION
from django.db import models, IntegrityError
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible


from .fields import URLSchemeField
from .settings import REGISTRY_TOKEN_TYPE, TOKEN_TYPE_BEARER


if VERSION >= (1, 5):
    from django.contrib.auth import get_user_model
    User = get_user_model()


@python_2_unicode_compatible
class Scope(models.Model):

    STATUS_ENABLED = 1
    STATUS_DISABLED = 2

    STATUS_CHOICES = (
        (STATUS_ENABLED, _('Enabled')),
        (STATUS_DISABLED, _('Disabled')),
    )

    identifier = models.CharField(_('Scope ID'), max_length=100, help_text=_('Scope identifier. Usually in form of `app_name:view_name`.'), unique=True)
    title = models.CharField(_('Scope title'), max_length=250, help_text=_('Scope human-friendly name.'))
    status = models.PositiveIntegerField(_('Status'), db_index=True, choices=STATUS_CHOICES, default=STATUS_ENABLED)

    class Meta:
        verbose_name = _('Scope')
        verbose_name_plural = _('Scopes')
        ordering = ['title']

    def __str__(self):
        return '%s' % self.identifier


@python_2_unicode_compatible
class Client(models.Model):

    TYPE_CONFIDENTIAL = 1
    '''
    Clients capable of maintaining the confidentiality of their
    credentials (e.g., client implemented on a secure server with
    restricted access to the client credentials), or capable of secure
    client authentication using other means.

    Examples:

    web application
      A web application is a confidential client running on a web
      server.  Resource owners access the client via an HTML user
      interface rendered in a user-agent on the device used by the
      resource owner.  The client credentials as well as any access
      token issued to the client are stored on the web server and are
      not exposed to or accessible by the resource owner.

    '''

    TYPE_PUBLIC = 2
    '''
    Clients incapable of maintaining the confidentiality of their
    credentials (e.g., clients executing on the device used by the
    resource owner, such as an installed native application or a web
    browser-based application), and incapable of secure client
    authentication via any other means.

    Examples:

    user-agent-based application
      A user-agent-based application is a public client in which the
      client code is downloaded from a web server and executes within a
      user-agent (e.g., web browser) on the device used by the resource
      owner.  Protocol data and credentials are easily accessible (and
      often visible) to the resource owner.  Since such applications
      reside within the user-agent, they can make seamless use of the
      user-agent capabilities when requesting authorization.

    native application
      A native application is a public client installed and executed on
      the device used by the resource owner.  Protocol data and
      credentials are accessible to the resource owner.  It is assumed
      that any client authentication credentials included in the
      application can be extracted.  On the other hand, dynamically
      issued credentials such as access tokens or refresh tokens can
      receive an acceptable level of protection.  At a minimum, these
      credentials are protected from hostile servers with which the
      application may interact.  On some platforms, these credentials
      might be protected from other applications residing on the same
      device.

    '''

    TYPE_CHOICES = (
        (TYPE_CONFIDENTIAL, _('Confidential')),
        (TYPE_PUBLIC, _('Public')),
    )

    date_registered = models.DateTimeField(_('Registered at'), auto_now_add=True)
    title = models.CharField(_('Title'), max_length=100, unique=True)
    user = models.ForeignKey(User, verbose_name=_('Registrant'), help_text=_('User who registered this client.'))
    description = models.TextField(_('Description'), max_length=100)
    link = models.URLField(_('URL'), help_text=_('Application webpage URL.'), null=True, blank=True)
    identifier = models.CharField(_('Identifier'), max_length=250, help_text=_('Public client identifier. <i>Generated automatically if empty.</i>.'), unique=True, blank=True)
    token_lifetime = models.IntegerField(_('Token lifetime'), help_text=_('Time in seconds after which token given to the application expires.'), null=True, blank=True)
    password = models.CharField(_('Password'), max_length=250, help_text=_('Secret that can be used along with an identifier as username to authenticate with HTTP Basic scheme.'), blank=True)
    type = models.IntegerField(_('Type'),
       help_text=_('<b>Confidential</b> &#8212; Clients capable of maintaining the confidentiality of their credentials, or capable of secure client authentication using other means.<br /> \
                  <b>Public</b> &#8212; Clients incapable of maintaining the confidentiality of their credentials, and incapable of secure client authentication via any other means'),
       choices=TYPE_CHOICES, default=TYPE_CONFIDENTIAL)
    scopes = models.ManyToManyField(Scope, verbose_name=_('Scopes'), help_text=_('The scopes client is restricted to. <i>All registered scopes will be available for the client if none selected.</i>'), null=True, blank=True)
    hash_sign_supported = models.BooleanField(_('Supports # in "Location"'), help_text=_('Should be checked if this client supports fragment component (#) in the HTTP "Location" response header field'), default=True)

    class Meta:
        verbose_name = _('Client')
        verbose_name_plural = _('Clients')

    def is_public(self):
        return self.type == self.TYPE_PUBLIC

    def is_confidential(self):
        return self.type == self.TYPE_CONFIDENTIAL

    def __str__(self):
        return '%s' % self.title

    def generate_indentifier(self):
        """Identifier length: 32 chars."""
        return str(uuid4()).replace('-', '')

    def save(self, force_insert=False, force_update=False, **kwargs):
        if self.identifier == '':
            while True:
                self.identifier = self.generate_indentifier()
                try:
                    super(Client, self).save(force_insert, force_update, **kwargs)
                except IntegrityError:
                    pass
                else:
                    break
        else:
            super(Client, self).save(force_insert, force_update, **kwargs)


@python_2_unicode_compatible
class RedirectionEndpoint(models.Model):
    """
    SPEC:

       The authorization server SHOULD require all clients to register their
       redirection endpoint prior to utilizing the authorization endpoint.

       The authorization server MAY allow the client to register multiple
       redirection endpoints.
    """

    client = models.ForeignKey(Client, verbose_name=_('Client'), related_name='redirection_uris')

    '''
    SPEC:

       The redirection endpoint URI MUST be an absolute URI as defined by
       [RFC3986] Section 4.3.  The endpoint URI MAY include an
       "application/x-www-form-urlencoded" formatted (per Appendix B) query
       component ([RFC3986] Section 3.4), which MUST be retained when adding
       additional query parameters.  The endpoint URI MUST NOT include a
       fragment component.

       If requiring the
       registration of the complete redirection URI is not possible, the
       authorization server SHOULD require the registration of the URI
       scheme, authority, and path (allowing the client to dynamically vary
       only the query component of the redirection URI when requesting
       authorization).

       Lack of a redirection URI registration requirement can enable an
       attacker to use the authorization endpoint as an open redirector as
       described in Section 10.15.

    '''
    uri = URLSchemeField(_('URI'), help_text=_('URI or URI scheme for authorization server to redirect client when an interaction with a resource owner is complete.'))

    class Meta:
        verbose_name = _('Redirection Endpoint')
        verbose_name_plural = _('Redirection Endpoints')

    def __str__(self):
        return '%s' % self.uri


@python_2_unicode_compatible
class AuthorizationCode(models.Model):

    # A maximum authorization code lifetime of 10 minutes is RECOMMENDED
    date_issued = models.DateTimeField(_('Issued at'), auto_now_add=True)
    code = models.CharField(_('Code'), max_length=7, help_text=_('Code issued upon authorization.'), unique=True)
    user = models.ForeignKey(User, verbose_name=_('User'), help_text=_('The user authorization is granted for.'))
    client = models.ForeignKey(Client, verbose_name=_('Client'), help_text=_('The client authorization is granted for.'))
    uri = URLSchemeField(_('Redirect URI'), help_text=_('The URI authorization is bound to.'))
    scopes = models.ManyToManyField(Scope, verbose_name=_('Scopes'), help_text=_('The scopes token issued with this code should be restricted to.'), null=True, blank=True)

    class Meta:
        verbose_name = _('Authorization code')
        verbose_name_plural = _('Authorization codes')

    def __str__(self):
        return '%s' % self.code

    def generate_code(self):
        """Code length: 7 chars."""
        return randrange(1000000, 9999999)

    def save(self, force_insert=False, force_update=False, **kwargs):
        if self.code == '':
            while True:
                self.code = self.generate_code()
                try:
                    super(AuthorizationCode, self).save(force_insert, force_update, **kwargs)
                except IntegrityError:
                    pass
                else:
                    break
        else:
            super(AuthorizationCode, self).save(force_insert, force_update, **kwargs)


@python_2_unicode_compatible
class Token(models.Model):

    # A maximum authorization code lifetime of 10 minutes is RECOMMENDED
    date_issued = models.DateTimeField(_('Issued at'), auto_now_add=True)
    expires_at = models.DateTimeField(_('Expires at'), null=True, blank=True)
    access_token = models.CharField(_('Access Token'), max_length=32, help_text=_('Token to be used to access resources.'), unique=True)
    refresh_token = models.CharField(_('Refresh Token'), max_length=32, help_text=_('Token to be used to refresh access token.'), unique=True, null=True, blank=True)
    access_token_type = models.CharField(_('Type'), max_length=100, help_text=_('Access token type client uses to apply the appropriate authorization method.'), choices=[(t[0], t[1]) for t in REGISTRY_TOKEN_TYPE], default=TOKEN_TYPE_BEARER)
    user = models.ForeignKey(User, verbose_name=_('User'), help_text=_('The user token is issued for.'), null=True, blank=True)
    client = models.ForeignKey(Client, verbose_name=_('Client'), help_text=_('The client application token is issued for.'))
    code = models.ForeignKey(AuthorizationCode, verbose_name=_('Code'), help_text=_('Authorization code used to generate this token.'), null=True, blank=True)
    scopes = models.ManyToManyField(Scope, verbose_name=_('Scopes'), help_text=_('The scopes token is restricted to.'), null=True, blank=True)

    class Meta:
        verbose_name = _('Token')
        verbose_name_plural = _('Tokens')

    def __str__(self):
        return '%s' % self.code

    def generate_token(self):
        """Identifier length: 32 chars."""
        return str(uuid4()).replace('-', '')

    def save(self, force_insert=False, force_update=False, **kwargs):
        if self.access_token == '':
            while True:
                self.access_token = self.generate_token()
                self.refresh_token = self.generate_token()

                try:
                    super(Token, self).save(force_insert, force_update, **kwargs)
                except IntegrityError:
                    pass
                else:
                    break
        else:
            super(Token, self).save(force_insert, force_update, **kwargs)



import json
from time import time
from base64 import b64decode
from datetime import datetime

from django.conf import settings
from django.contrib.auth import SESSION_KEY
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.utils.translation import ugettext_lazy as _

from django import VERSION

if VERSION >= (1, 5):
    from django.contrib.auth import get_user_model
    User = get_user_model()
else:
    from django.contrib.auth.models import User

from .exceptions import EndpointError, OauthostException
from .models import Client, AuthorizationCode, Token, Scope
from .settings import TEMPLATE_AUTHORIZE, TEMPLATE_AUTHORIZE_ERROR, TEMPLATE_AUTHORIZE_PROCEED
from .utils import get_remote_ip, LOGGER


class ScopeException(OauthostException):
    """Exceptions raised if scope conflict occures."""

    def __init__(self, message):
        self.message = message


class ErrorOauthostPage(EndpointError):
    """This error is able to be rendered as oauthost page."""

    def __init__(self, message, request, http_status=400):
        self.request = request
        self.message = message
        self.http_status = http_status

    def as_response(self):
        """For authorization endpoint. Renders a page with error description."""
        data_dict = {'oauthost_title': _('Error'), 'oauthost_error_text': self.message}
        return render(self.request, TEMPLATE_AUTHORIZE_ERROR, data_dict, status=self.http_status)


class ErrorTokenEndpointRedirect(EndpointError):
    """This error is able to perform a redirect from Token Endpoint."""

    def __init__(self, error_type, message, http_status=400, additional_headers=None):
        self.error_type = error_type
        self.message = message
        self.http_status = http_status
        if additional_headers is None:
            additional_headers = {}
        self.additional_headers = additional_headers

    def as_response(self):
        """For token endpoint. Issues JSON error response."""
        return TokenEndpoint.build_response({'error': self.error_type, 'error_description': self.message}, self.http_status, self.additional_headers)


class ErrorAuthorizeEndpointRedirect(EndpointError):
    """This error is able to perform a redirect from Authorize Endpoint."""

    def __init__(self, error_type, message, redirect_uri, state, uri_fragment_supported=False):
        self.error_type = error_type
        self.message = message
        self.redirect_uri = redirect_uri
        self.state = state
        self.uri_fragment_supported = uri_fragment_supported

    def as_response(self):
        """For authorization endpoint. Issues error response."""
        doc = {'error': self.error_type, 'error_description': self.message}
        if self.state is not None:
            doc.update({'state': self.state})
        return HttpResponseRedirect(AuthorizeEndpoint.build_redirect_url(self.redirect_uri, doc, self.uri_fragment_supported))


class EndpointBase(object):
    """Basic class for endpoint classes."""

    _input_params_source = 'POST'
    state = None

    def __init__(self, request):
        self.request = request
        self.input_params = self.filter_input_params(getattr(self.request, self._input_params_source))

    @classmethod
    def filter_input_params(cls, input_params):
        """Filters request parameters and returns filtered dictionary.

        SPEC: Parameters sent without a value MUST be treated as if they were omitted from the request.

        """
        params_filtered = {}
        for key, value in input_params.items():
            if value:
                params_filtered[key] = value
        return params_filtered

    def filter_scopes(self, client):
        """Gets space delimited list of scopes from client request,
        and returns a list of scope objects, corrected according
        to auth server settings.

        SPEC:

            The authorization server MAY fully or partially ignore the scope
            requested by the client, based on the authorization server policy or
            the resource owner's instructions.  If the issued access token scope
            is different from the one requested by the client, the authorization
            server MUST include the "scope" response parameter to inform the
            client of the actual scope granted.

            If the client omits the scope parameter when requesting
            authorization, the authorization server MUST either process the
            request using a pre-defined default value or fail the request
            indicating an invalid scope.

        """
        scopes_requested = self.input_params.get('scope')  # OPTIONAL

        if scopes_requested is not None:
            # We already have scope objects.
            if isinstance(scopes_requested[0], Scope):
                return scopes_requested
            scopes_requested = scopes_requested.split(' ')
        else:
            scopes_requested = []

        client_scopes = []
        scopes_filtered = []
        scopes_available = []

        # Scopes associated with the client.
        for scope in client.scopes.filter(status=Scope.STATUS_ENABLED).all():
            client_scopes.append(scope)

        # No scopes requested, and client is unrestricted by scopes.
        # Prevent accessing all scopes.
        if not scopes_requested and not client_scopes:
            raise ScopeException('Scope must be provided for this client.')

        if client_scopes:
            scopes_available = client_scopes
        else:
            # Client is not restricted by any scopes, so we make all scopes available.
            for scope in Scope.objects.filter(status=Scope.STATUS_ENABLED).all():
                scopes_available.append(scope)

        if scopes_requested and not scopes_available:
            raise ScopeException('Server doesn\'t accept scope.')

        # Index scopes by ID.
        scopes_available_ = {}
        for scope in scopes_available:
            scopes_available_[scope.identifier] = scope
        scopes_available = scopes_available_
        del scopes_available_

        scopes_available_set = set(scopes_available.keys())
        unsupported = set(scopes_requested).difference(scopes_available_set)
        if unsupported:
            raise ScopeException('Unsupported scope requested: %s.' % ' '.join(unsupported))

        for scope_name in set(scopes_requested).intersection(scopes_available_set):
            scopes_filtered.append(scopes_available[scope_name])

        return scopes_filtered

    @classmethod
    def apply_scopes(cls, auth_obj, filtered_scopes):
        """Link scopes to an auth object: Token or AuthorizationCode."""
        if auth_obj is not None:
            for scope in filtered_scopes:
                auth_obj.scopes.add(scope)

    def check_https(self):
        """Performs an HTTPS check.
        Insecure connections are only available in debug mode.

        """
        if not self.request.is_secure() and not settings.DEBUG:
            raise ErrorOauthostPage(_('OAuth 2.0 requires secure connection.'), self.request, http_status=403)

    def get_response(self):
        """Returns an endpoint response."""
        try:
            self.check_https()
            self.state = self.input_params.get('state')  # OPTIONAL
            response = self.process_request()
        except EndpointError as e:
            return e.as_response()

        return response

    @classmethod
    def generate_token(cls, client, user, token_obj_params=None):
        """Returns an access token."""
        expires_in = client.token_lifetime
        expires_at = None
        if expires_in is not None:
            expires_at = datetime.fromtimestamp(int(time() + expires_in))
        if token_obj_params is None:
            token_obj_params = {}
        return Token(client=client, user=user, expires_at=expires_at, **token_obj_params)

    @classmethod
    def build_token_document(cls, token, expires_in=None, with_refresh_token=True):
        """Returns a token document as a dict."""
        output_params = {
            'access_token': token.access_token,
            'token_type': token.access_token_type
        }

        if token.scopes:
            output_params['scope'] = ' '.join([scope.identifier for scope in token.scopes.all()])

        if expires_in is not None:
            output_params['expires_in'] = expires_in

        if with_refresh_token:
            output_params['refresh_token'] = token.refresh_token

        return output_params

    def get_client(self):
        """Returns client object."""
        raise NotImplementedError()

    def process_request(self):
        """Main method performing client request processing."""
        raise NotImplementedError()


class TokenEndpoint(EndpointBase):
    """
    SPEC:
        The token endpoint is used by the client to obtain an access token by
        presenting its authorization grant or refresh token.  The token
        endpoint is used with every authorization grant except for the
        implicit grant type (since an access token is issued directly).

        Since requests to the token endpoint result in the transmission of
        clear-text credentials (in the HTTP request and response), the
        authorization server MUST require the use of TLS as described in
        Section 1.6 when sending requests to the token endpoint.

        The client MUST use the HTTP "POST" method when making access token
        requests.

        The authorization and token endpoints allow the client to specify the
        scope of the access request using the "scope" request parameter.  In
        turn, the authorization server uses the "scope" response parameter to
        inform the client of the scope of the access token issued.

    PARAMETERS:

        SUCCESS
            access_token REQUIRED
            token_type REQUIRED
            expires_in RECOMMENDED
            refresh_token OPTIONAL
            scope OPTIONAL

        FAILURE
            error REQUIRED
                invalid_request
                invalid_client
                invalid_grant
                unauthorized_client
                unsupported_grant_type
                invalid_scope
            error_description OPTIONAL
            error_uri OPTIONAL

    authorization_code
    ------------------
        REQUEST
            grant_type REQUIRED
            code REQUIRED
            redirect_uri REQUIRED, if the "redirect_uri" was included in the auth request values MUST be identical.
            client_id REQUIRED

    password
    --------
        REQUEST
            grant_type REQUIRED
            username REQUIRED
            password REQUIRED
            scope OPTIONAL

    client_credentials
    ------------------
        REQUEST
            grant_type REQUIRED
            scope OPTIONAL

        SUCCESS
            ** DROP refresh_token

    refresh_token
    -------------
        REQUEST
            grant_type REQUIRED
            refresh_token REQUIRED
            scope OPTIONAL

    """

    ERROR_INVALID_REQUEST = 'invalid_request'
    ERROR_INVALID_CLIENT = 'invalid_client'
    ERROR_INVALID_GRANT = 'invalid_grant'
    ERROR_UNAUTHORIZED_CLIENT = 'unauthorized_client'
    ERROR_UNSUPPORTED_GRANT_TYPE = 'unsupported_grant_type'
    ERROR_INVALID_SCOPE = 'invalid_scope'

    _allowed_grant_types = ('authorization_code', 'password', 'client_credentials', 'refresh_token')

    @classmethod
    def build_response(cls, data_dict, http_status=200, additional_headers=None):
        """For token endpoint. Issues JSON response."""
        response = HttpResponse(content_type='application/json;charset=UTF-8', content=json.dumps(data_dict), status=http_status)
        response['Cache-Control'] = 'no-store'
        response['Pragma'] = 'no-cache'
        if additional_headers is None:
            additional_headers = {}
        for key, value in additional_headers.items():
            response[key] = value
        return response

    def get_client(self):
        """Returns client object."""
        client = None
        auth_error_headers = {}
        client_id = None
        client_password = None

        authorization_method = self.request.META.get('Authorization')
        if authorization_method is not None:  # Authorization header detected.
            auth_method_type, auth_method_value = authorization_method.split(' ', 1)
            auth_error_headers['WWW-Authenticate'] = auth_method_type
            # Handle client auth through HTTP Basic.
            if auth_method_type == 'Basic':
                try:
                    client_id, client_password = b64decode(auth_method_value).decode('utf-8').split(':')
                except Exception:
                    pass
        else:
            '''
            SPEC:
               Alternatively, the authorization server MAY support including the
               client credentials in the request-body using the following
               parameters: client_id, client_secret.

               Including the client credentials in the request-body using the two
               parameters is NOT RECOMMENDED and SHOULD be limited to clients unable
               to directly utilize the HTTP Basic authentication scheme (or other
               password-based HTTP authentication schemes).  The parameters can only
               be transmitted in the request-body and MUST NOT be included in the
               request URI.

               The parameters can only
               be transmitted in the request-body and MUST NOT be included in the
               request URI.
            '''
            client_id = self.input_params.get('client_id')
            client_password = self.input_params.get('client_secret')

        if client_id is not None:
            try:
                client = Client.objects.get(identifier=client_id)
            except ObjectDoesNotExist:
                client = None

            '''
            SPEC:

               A client MAY use the "client_id" request parameter to identify itself
               when sending requests to the token endpoint.  In the
               "authorization_code" "grant_type" request to the token endpoint, an
               unauthenticated client MUST send its "client_id" to prevent itself
               from inadvertently accepting a code intended for a client with a
               different "client_id".  This protects the client from substitution of
               the authentication code.  (It provides no additional security for the
               protected resource.)

            '''
            if client is not None:
                if client.password.strip() != '' and client.password != client_password:
                    client = None

        if client is None:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_CLIENT, 'Unable to authenticate client by its credentials.', http_status=401, additional_headers=auth_error_headers)

        return client

    def get_token_for_grant_type(self, grant_type, client):
        """Returns token object for a given grant type."""

        filtered_scopes = []
        if grant_type not in ('refresh_token', 'authorization_code'):  # For refresh we use initially granted scope.
            try:
                filtered_scopes = self.filter_scopes(client)
            except ScopeException as e:
                raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_SCOPE, e.message)

        method = getattr(self, 'handle_%s' % grant_type)
        token = method(client)
        token.save()
        self.apply_scopes(token, filtered_scopes)

        return token

    def handle_authorization_code(self, client):
        """Access Token Request"""

        code = self.input_params.get('code')  # REQUIRED
        redirect_uri = self.input_params.get('redirect_uri')  # REQUIRED

        if code is None or redirect_uri is None:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_REQUEST, 'Required param(s) are missing. Expected: `code` and `redirect_uri`.')

        try:
            code = AuthorizationCode.objects.get(code=code)
        except ObjectDoesNotExist:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Invalid authorization code is supplied.')

        '''
        SPEC:

            If an authorization code is used more than
            once, the authorization server MUST deny the request and SHOULD
            revoke (when possible) all tokens previously issued based on
            that authorization code.  The authorization code is bound to
            the client identifier and redirection URI.

        '''
        previous_tokens = Token.objects.filter(code=code).all()
        if len(previous_tokens) > 0:
            previous_tokens.delete()
            code.delete()
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Authorization code is used more than once. Code and tokens are revoked.')

        if code.uri != redirect_uri:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Supplied URI does not match the URI associated with authorization code.')

        if code.client.id != client.id:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Authorization code supplied was issued to another client.')

        token = self.generate_token(client, code.user, {'code': code})
        token.save()

        # Copying scopes from code.
        for scope in code.scopes.all():
            token.scopes.add(scope)

        return token

    def handle_client_credentials(cls, client):
        """
        SPEC:

        Client Credentials Grant

            The client can request an access token using only its client
            credentials (or other supported means of authentication) when the
            client is requesting access to the protected resources under its
            control, or those of another resource owner that have been previously
            arranged with the authorization server (the method of which is beyond
            the scope of this specification).

           The client credentials grant type MUST only be used by confidential
           clients.

        """

        if not client.is_confidential():
            raise ErrorTokenEndpointRedirect('client_credentials', 'This client type is not authorized to use this grant type.')

        # Let's suppose that the user is the one, who has registered the client.
        return cls.generate_token(client, client.user)

    def handle_password(self, client):
        """
        SPEC:

        Resource Owner Password Credentials Grant

           The resource owner password credentials grant type is suitable in
           cases where the resource owner has a trust relationship with the
           client, such as the device operating system or a highly privileged
           application.  The authorization server should take special care when
           enabling this grant type and only allow it when other flows are not
           viable.

           This grant type is suitable for clients capable of obtaining the
           resource owner's credentials (username and password, typically using
           an interactive form).  It is also used to migrate existing clients
           using direct authentication schemes such as HTTP Basic or Digest
           authentication to OAuth by converting the stored credentials to an
           access token.

           The authorization server and client SHOULD minimize use of this grant
           type and utilize other grant types whenever possible.

        """
        username = self.input_params.get('username')  # REQUIRED
        password = self.input_params.get('password')  # REQUIRED

        if username is None or password is None:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_REQUEST, 'Required param(s) are missing. Expected: `username` and `password`.')

        try:
            user = User.objects.get(username=username)
            if user.check_password(password):
                return self.generate_token(client, user)
        except ObjectDoesNotExist:
            pass

        raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Supplied resource owner credentials are invalid.')

    def handle_refresh_token(self, client):
        """Refreshes an Access Token."""
        refresh_token = self.input_params.get('refresh_token')

        if refresh_token is None:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_REQUEST, 'Required `refresh_token` param is missing.')

        try:
            token = Token.objects.get(refresh_token=refresh_token)
        except ObjectDoesNotExist:
            raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Refresh token supplied is invalid.')
        else:
            if token.client_id != client.id:
                raise ErrorTokenEndpointRedirect(self.ERROR_INVALID_GRANT, 'Refresh token supplied was issued to another client.')

        # For refresh token grant we only swap token values.
        token.date_issued = datetime.now()
        token.access_token = token.generate_token()
        token.refresh_token = token.generate_token()
        return token

    def process_request(self):
        """Main method performing client request processing."""
        grant_type = self.input_params.get('grant_type')  # REQUIRED
        if grant_type not in self._allowed_grant_types:
            raise ErrorTokenEndpointRedirect(self.ERROR_UNSUPPORTED_GRANT_TYPE, 'Unsupported grant type requested')

        client = self.get_client()
        token = self.get_token_for_grant_type(grant_type, client)
        token_doc = self.build_token_document(token, client.token_lifetime, with_refresh_token=(grant_type != 'client_credentials'))

        return self.build_response(token_doc)


class AuthorizeEndpoint(EndpointBase):
    """
    SPEC:
        The authorization endpoint is used to interact with the resource
        owner and obtain an authorization grant.  The authorization server
        MUST first verify the identity of the resource owner.  The way in
        which the authorization server authenticates the resource owner
        (e.g., username and password login, session cookies) is beyond the
        scope of this specification.

        Since requests to the authorization endpoint result in user
        authentication and the transmission of clear-text credentials (in the
        HTTP response), the authorization server MUST require the use of TLS
        as described in Section 1.6 when sending requests to the
        authorization endpoint.

        The authorization server MUST support the use of the HTTP "GET"
        method [RFC2616] for the authorization endpoint and MAY support the
        use of the "POST" method as well.

        The authorization and token endpoints allow the client to specify the
        scope of the access request using the "scope" request parameter.  In
        turn, the authorization server uses the "scope" response parameter to
        inform the client of the scope of the access token issued.

    PARAMETERS:

        REQUEST
            response_type REQUIRED
            client_id REQUIRED
            redirect_uri OPTIONAL
            scope OPTIONAL
            state RECOMMENDED

        FAILURE
            error REQUIRED
                invalid_request
                unauthorized_client
                access_denied
                unsupported_response_type
                invalid_scope
                server_error
                temporarily_unavailable
            error_description OPTIONAL
            error_uri OPTIONAL
            state REQUIRED if set

    code
    ----
        SUCCESS
            code REQUIRED
            state REQUIRED if set

    token
    -----
        SUCCESS
            access_token REQUIRED
            token_type REQUIRED
            expires_in RECOMMENDED
            scope OPTIONAL; if not identical to the scope requested by the client REQUIRED
            state REQUIRED if set

    """

    ERROR_INVALID_REQUEST = 'invalid_request'
    ERROR_UNAUTHORIZED_CLIENT = 'unauthorized_client'
    ERROR_ACCESS_DENIED = 'access_denied'
    ERROR_UNSUPPORTED_RESPONSE_TYPE = 'unsupported_response_type'
    ERROR_INVALID_SCOPE = 'invalid_scope'
    ERROR_SERVER_ERROR = 'server_error'
    ERROR_TEMPORARILY_UNAVAILABLE = 'temporarily_unavailable'

    _input_params_source = 'REQUEST'
    _allowed_response_types = ('code', 'token')

    def get_client(self):
        """Returns client object."""
        client_id = self.input_params.get('client_id')  # REQUIRED

        if client_id is None:
            # Fail fast without a DB hit.
            raise ErrorOauthostPage(_('Client ID must be supplied.'), self.request)

        try:
            client = Client.objects.get(identifier=client_id)
        except ObjectDoesNotExist:
            LOGGER.error('Invalid client ID supplied. Value "%s" was sent from IP "%s".' % (client_id, get_remote_ip(self.request)))
            raise ErrorOauthostPage(_('Invalid client ID is supplied.'), self.request)

        return client

    def get_redirect_url(self, client):
        """Calculates a redirect URI.
        Returns a tuple: uri_from_request, uri_fixed_server

        """
        input_uri = self.input_params.get('redirect_uri')  # OPTIONAL
        actual_uri = input_uri

        registered_uris = [url[0] for url in client.redirection_uris.values_list('uri')]

        # Check redirection URI validity.
        if input_uri is None:
            # redirect_uri is optional and was not supplied.
            if len(registered_uris) == 1:
                # There is only one URI associated with client, so we use it.
                actual_uri = registered_uris[0]
            else:
                # Several URIs are registered with the client, decision is ambiguous.
                LOGGER.error('Redirect URI was not supplied by client with ID "%s". Request from IP "%s".' % (client.id, get_remote_ip(self.request)))
                raise ErrorOauthostPage(_('Redirect URI should be supplied for a given client.'), self.request)

        '''
        SPEC:

            The authorization server SHOULD NOT redirect the user-agent to unregistered or untrusted URIs
            to prevent the authorization endpoint from being used as an open redirector.
        '''
        if actual_uri not in registered_uris:
            LOGGER.error('An attempt to use an untrusted URI "%s" for client with ID "%s". Request from IP "%s".' % (actual_uri, client.id, get_remote_ip(self.request)))
            raise ErrorOauthostPage(_('Redirection URI supplied is not associated with given client.'), self.request)

        return actual_uri

    def render_scopes_page(self, client, scopes):
        """Returns a response with oauthost page listing requested scopes."""
        dict_data = {
            'client': client,
            'scopes_obj': scopes,
            'oauthost_title': _('Authorization Request')
        }
        return render(self.request, TEMPLATE_AUTHORIZE, dict_data)

    def request_auth_confirmation(self):
        client = self.get_client()
        redirect_uri = self.get_redirect_url(client)

        state = self.input_params.get('state')  # RECOMMENDED

        response_type = self.input_params.get('response_type')  # REQUIRED
        if response_type not in self._allowed_response_types:
            raise ErrorAuthorizeEndpointRedirect(self.ERROR_UNSUPPORTED_RESPONSE_TYPE, 'Unsupported response type requested', redirect_uri, state)

        try:
            filtered_scopes = self.filter_scopes(client)
        except ScopeException as e:
            raise ErrorAuthorizeEndpointRedirect(self.ERROR_INVALID_SCOPE, e.message, redirect_uri, state)

        self.request.session['oauth_client_id'] = client.id
        self.request.session['oauth_response_type'] = response_type
        self.request.session['oauth_redirect_uri'] = redirect_uri
        self.request.session['oauth_scopes_ids'] = [s.id for s in filtered_scopes]
        self.request.session['oauth_state'] = state

        return self.render_scopes_page(client, filtered_scopes)

    def _clear_session(self):
        """For authorization endpoint. Clears oauth data from session."""
        self.request.session[SESSION_KEY] = None

    @classmethod
    def build_redirect_url(cls, redirect_base, params, use_uri_fragment):
        """For authorization endpoint. Builds up redirection URL."""
        if use_uri_fragment:
            redirect_base = '%s#' % redirect_base
        else:
            # SPEC: query component MUST be retained when adding additional query parameters.
            if redirect_base is None or '?' not in redirect_base:
                redirect_base = '%s?' % redirect_base
        return '%s%s' % (redirect_base, '&'.join(['%s=%s' % (key, value) for key, value in params.items()]))

    def get_response_document(self, response_type, redirect_uri, client, params_as_uri_fragment, state):
        """Returns a response object for a given response type."""
        try:
            filtered_scopes = self.filter_scopes(client)
        except ScopeException as e:
            raise ErrorAuthorizeEndpointRedirect(self.ERROR_INVALID_SCOPE, e.message, redirect_uri, state)

        method = getattr(self, 'handle_%s' % response_type)

        output_params = method(client, redirect_uri, filtered_scopes)

        if state is not None:
            output_params['state'] = state

        '''
        SPEC:

           Developers should note that some user-agents do not support the
           inclusion of a fragment component in the HTTP "Location" response
           header field.  Such clients will require using other methods for
           redirecting the client than a 3xx redirection response -- for
           example, returning an HTML page that includes a 'continue' button
           with an action linked to the redirection URI.
        '''
        if not client.hash_sign_supported:
            data_dict = {'action_uri': self.build_redirect_url(redirect_uri, output_params, params_as_uri_fragment)}
            return render(self.request, TEMPLATE_AUTHORIZE_PROCEED, data_dict)

        return HttpResponseRedirect(self.build_redirect_url(redirect_uri, output_params, params_as_uri_fragment))

    def handle_code(self, client, redirect_uri, filtered_scopes):
        """
        SPEC:

        Authorization Code Grant

           The authorization code grant type is used to obtain both access
           tokens and refresh tokens and is optimized for confidential clients.
           Since this is a redirection-based flow, the client must be capable of
           interacting with the resource owner's user-agent (typically a web
           browser) and capable of receiving incoming requests (via redirection)
           from the authorization server.

        """
        code = AuthorizationCode(client=client, user=self.request.user, uri=redirect_uri)
        code.save()
        self.apply_scopes(code, filtered_scopes)

        return {'code': code.code}

    def handle_token(self, client, redirect_uri, filtered_scopes):
        """
        SPEC:

        Implicit Grant

            The implicit grant type is used to obtain access tokens (it does not
            support the issuance of refresh tokens) and is optimized for public
            clients known to operate a particular redirection URI.  These clients
            are typically implemented in a browser using a scripting language
            such as JavaScript.

        """
        token = self.generate_token(client, self.request.user)
        token.save()
        self.apply_scopes(token, filtered_scopes)

        return self.build_token_document(token, client.token_lifetime, with_refresh_token=False)

    def process_request(self):
        """Main method performing client request processing.

        SPEC:
            The authorization server validates the request to ensure that all
            required parameters are present and valid.  If the request is valid,
            the authorization server authenticates the resource owner and obtains
            an authorization decision (by asking the resource owner or by
            establishing approval via other means).

            When a decision is established, the authorization server directs the
            user-agent to the provided client redirection URI using an HTTP
            redirection response, or by other means available to it via the
            user-agent.
        """
        if self.request.POST.get('auth_decision') is None:
            # Obtain an authorization decision by asking the resource owner.
            return self.request_auth_confirmation()

        # User has made his choice using auth form.
        client_id = self.request.session.get('oauth_client_id')
        response_type = self.request.session.get('oauth_response_type')
        redirect_uri = self.request.session.get('oauth_redirect_uri')
        scopes_ids = self.request.session.get('oauth_scopes_ids')
        state = self.request.session.get('oauth_state')

        params_as_uri_fragment = (response_type == 'token')

        if self.request.POST.get('confirmed') is None:
            # User has declined authorization.
            self._clear_session()
            raise ErrorAuthorizeEndpointRedirect(self.ERROR_ACCESS_DENIED, 'Authorization is canceled by user', redirect_uri, state, params_as_uri_fragment)

        # Simulate scope for filter_scopes()
        self.input_params['scope'] = Scope.objects.filter(id__in=scopes_ids).all()
        client = Client.objects.get(pk=client_id)

        response = self.get_response_document(response_type, redirect_uri, client, params_as_uri_fragment, state)
        self._clear_session()

        return response



from django import forms
from django.db import models
from django.conf import settings


# This allows South to handle our custom 'URLSchemeField' field.
if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules

    add_introspection_rules([], ['^oauthost\.fields\.URLSchemeField'])


class URLShemeFormField(forms.URLField):
    def __init__(self, *args, **kwargs):
        super(URLShemeFormField, self).__init__(*args, **kwargs)
        del self.validators[-1]


class URLSchemeField(models.URLField):
    def __init__(self, verbose_name=None, name=None, **kwargs):
        models.URLField.__init__(self, verbose_name, name, **kwargs)
        del self.validators[-1]

    def formfield(self, **kwargs):
        defaults = {'form_class': URLShemeFormField}
        defaults.update(kwargs)
        return super(URLSchemeField, self).formfield(**defaults)



from .settings import REGISTRY_TOKEN_TYPE
from .models import Client, Scope, RedirectionEndpoint
from .exceptions import OauthostException


def register_client(title, identifier, redirect_uri, registrant, scopes_list=None, register_unknown_scopes=True, token_lifetime=3600, public=True, client_params=None):
    """Registers a client.

    :param title: client title.
    :param identifier: client identifier
    :param redirect_uri: redirect URI to associate with this client
    :param registrant: user who registeres this client
    :param scopes_list: a list of scope identifiers or Scope objects to restrict the client to.
    :param register_unknown_scopes: this allows to raise OauthostException if scopes_list contans an unregistered item instead of registering it.
    :param token_lifetime: lifetime for tokens that'll issued to this client
    :param public: flag to allow, switching from public client type to confidential
    :param client_params: arbitary parameters to build a Client object from
    :return: client object
    :rtype: Client
    """
    client_kwargs = {}

    if client_params is not None:
        client_kwargs.update(client_params)

    client_kwargs.update({
        'title': title,
        'identifier': identifier,
        'user': registrant,
        'token_lifetime': token_lifetime,
    })

    if public:
        client_kwargs['type'] = Client.TYPE_PUBLIC
    else:
        client_kwargs['type'] = Client.TYPE_CONFIDENTIAL

    target_scope_objects = []
    if scopes_list:
        registered_scopes_ = Scope.objects.filter(identifier__in=scopes_list).all()
        registered_scopes = {}
        for scope in registered_scopes_:
            registered_scopes[scope.identifier] = scope
        del registered_scopes_

        for scope in scopes_list:
            scope_obj = scope
            if not isinstance(scope, Scope):
                if scope in registered_scopes:
                    scope_obj = registered_scopes[scope]
                else:
                    if register_unknown_scopes:
                        scope_obj = Scope(identifier=scope, title=scope)
                        scope_obj.save()
                    else:
                        raise OauthostException('Unable to register client to an unknown `%s` scope.' % scope)

            target_scope_objects.append(scope_obj)

    cl = Client(**client_kwargs)
    cl.save()

    for scope_obj in target_scope_objects:
        cl.scopes.add(scope_obj)

    if redirect_uri is not None:
        RedirectionEndpoint(uri=redirect_uri, client=cl).save()

    return cl


def auth_handler_response(request, scope=None):
    """Checks for token data in request using various
    methods depending on token types defined in REGISTRY_TOKEN_TYPE.

    ``scope`` - scope identifier string to check token has access to the scope.

    """
    token_auth_classes = [item[2] for item in REGISTRY_TOKEN_TYPE]

    for auth_class in token_auth_classes:
        handler = auth_class(request, scope)
        response = handler.response()
        if response is not None:
            return response

    return None


class PistonAuthHelper(object):
    """Authentication class for Piston resources.

    To be used in a usual piston-auth-way::

        from piston.resource import Resource
        from oauthost.utils import PistonAuthHelper

        my_resource_view = Resource(MyResourceHandler, authentication=PistonAuthHelper('my_resource:my_scope'))

    """

    def __init__(self, target_scope):
        self.target_scope = target_scope

    def is_authenticated(self, request):
        self.auth_response = auth_handler_response(request, scope=self.target_scope)
        return self.auth_response is None

    def challenge(self):
        return self.auth_response



from django.contrib import admin

from .models import Client, RedirectionEndpoint, AuthorizationCode, Scope, Token


class AuthorizationCodeInlineAdmin(admin.TabularInline):
    model = AuthorizationCode
    extra = 1


class RedirectionEndpointAdmin(admin.TabularInline):
    model = RedirectionEndpoint
    extra = 1


class TokenInlineAdmin(admin.TabularInline):
    model = Token
    extra = 1


class ClientAdmin(admin.ModelAdmin):
    list_display = ('title',)
    list_display_links = ('title',)
    search_fields = ('title',)
    inlines = (RedirectionEndpointAdmin, AuthorizationCodeInlineAdmin, TokenInlineAdmin)


class ScopeAdmin(admin.ModelAdmin):
    list_display = ('title', 'identifier', 'status')
    list_display_links = ('title',)
    search_fields = ('title', 'identifier')
    list_filter = ('status',)


class TokenAdmin(admin.ModelAdmin):
    list_display = ('date_issued', 'user', 'access_token', 'access_token_type', 'refresh_token')
    search_fields = ('access_token', 'refresh_token')
    list_filter = ('access_token_type', 'client')
    ordering = ('-date_issued',)
    date_hierarchy = 'date_issued'


class AuthorizationCodeAdmin(admin.ModelAdmin):
    list_display = ('date_issued', 'code', 'client', 'user', 'uri')
    list_display_links = ('code',)
    search_fields = ('code', 'uri')
    list_filter = ('client',)
    ordering = ('-date_issued',)
    date_hierarchy = 'date_issued'


admin.site.register(Client, ClientAdmin)
admin.site.register(Scope, ScopeAdmin)
admin.site.register(Token, TokenAdmin)
admin.site.register(AuthorizationCode, AuthorizationCodeAdmin)



try:
    from django.conf.urls.defaults import patterns, url
except ImportError:
    # For Django 1.6
    from django.conf.urls import patterns, url


urlpatterns = patterns('oauthost',
    url(r'^auth/$', 'views.endpoint_authorize', name='oauthost_authorize'),
    # SPEC: The [token] endpoint URI MUST NOT include a fragment component.
    url(r'^token/$', 'views.endpoint_token', name='oauthost_token'),
)



import logging


LOGGER = logging.getLogger('django.oauthost')


def get_remote_ip(request):
    """Resolves and returns client IP."""

    forwarded = request.META.get('HTTP_X_FORWARDED_FOR')
    ip = request.META.get('REMOTE_ADDR')
    if forwarded is not None:
        ip = forwarded.split(',')[-1].strip()
    return ip



from django.conf import settings

from .auth_handlers import BearerAuthHandler


# Someday here might be something more than bare Bearer.
TOKEN_TYPE_BEARER = 'bearer'
REGISTRY_TOKEN_TYPE = {
    (TOKEN_TYPE_BEARER, 'Bearer', BearerAuthHandler),
}

TEMPLATE_AUTHORIZE = getattr(settings, 'OAUTHOST_TEMPLATE_AUTHORIZE', 'oauthost/authorize.html')
TEMPLATE_AUTHORIZE_ERROR = getattr(settings, 'OAUTHOST_TEMPLATE_AUTHORIZE_ERROR', 'oauthost/authorize_error.html')
TEMPLATE_AUTHORIZE_PROCEED = getattr(settings, 'OAUTHOST_TEMPLATE_AUTHORIZE_PROCEED', 'oauthost/authorize_proceed.html')
TEMPLATE_FORBIDDEN = getattr(settings, 'OAUTHOST_TEMPLATE_FORBIDDEN', 'oauthost/forbidden.html')
TEMPLATE_RESTRICTED = getattr(settings, 'OAUTHOST_TEMPLATE_RESTRICTED', 'oauthost/restricted.html')
